@@ -65,7 +65,7 @@ void hyperlink_to_event_tagid(int tagid){
 */
 void event_page(void){
   int rid = 0;             /* rid of the event artifact */
-  char *zUuid;             /* UUID corresponding to rid */
+  char *zUuid;             /* artifact hash corresponding to rid */
   const char *zId;         /* Event identifier */
   const char *zVerbose;    /* Value of verbose option */
   char *zETime;            /* Time of the tech-note */
@@ -112,10 +112,11 @@ void event_page(void){
     }
   }
   db_finalize(&q1);
+  style_set_current_feature("event");
   if( rid==0 || (specRid!=0 && specRid!=rid) ){
     style_header("No Such Tech-Note");
     @ Cannot locate a technical note called <b>%h(zId)</b>.
-    style_footer();
+    style_finish_page();
     return;
   }
   zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
@@ -213,7 +214,7 @@ void event_page(void){
     wiki_convert(&comment, 0, WIKI_INLINE);
     blob_reset(&comment);
     @ </div>
-    @ </blockquote><hr />
+    @ </blockquote><hr>
   }
 
   if( fossil_strcmp(zMimetype, "text/x-fossil-wiki")==0 ){
@@ -229,8 +230,9 @@ void event_page(void){
                        "  FROM tag"
                        " WHERE tagname GLOB 'event-%q*'",
                     zId);
-  attachment_list(zFullId, "<hr /><h2>Attachments:</h2><ul>");
-  style_footer();
+  attachment_list(zFullId, "<hr><h2>Attachments:</h2><ul>");
+  document_emit_js();
+  style_finish_page();
   manifest_destroy(pTNote);
 }
 
@@ -271,14 +273,14 @@ int event_commit_common(
   zETime[10] = 'T';
   blob_appendf(&event, "E %s %s\n", zETime, zId);
   zETime[10] = ' ';
+  if( zMimetype && zMimetype[0] ){
+    blob_appendf(&event, "N %s\n", zMimetype);
+  }
   if( rid ){
     char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
     blob_appendf(&event, "P %s\n", zUuid);
     free(zUuid);
   }
-  if( zMimetype && zMimetype[0] ){
-    blob_appendf(&event, "N %s\n", zMimetype);
-  }
   if( zClr && zClr[0] ){
     blob_appendf(&event, "T +bgcolor * %F\n", zClr);
   }
@@ -359,6 +361,10 @@ int event_commit_common(
 **    mimetype=TEXT     Mimetype for w= text
 **    newclr            Use a background color
 **    clr=TEXT          Background color to use if newclr
+**
+** For GET requests, when editing an existing technote newclr and clr
+** are implied if a custom color has been set on the previous version
+** of the technote.
 */
 void eventedit_page(void){
   char *zTag;
@@ -412,10 +418,20 @@ void eventedit_page(void){
     login_needed(g.anon.Write && (rid ? g.anon.WrWiki : g.anon.NewWiki));
     return;
   }
+  style_set_current_feature("event");
 
   /* Figure out the color */
   if( rid ){
     zClr = db_text("", "SELECT bgcolor FROM event WHERE objid=%d", rid);
+    if( zClr && zClr[0] ){
+      const char * zRequestMethod = P("REQUEST_METHOD");
+      if(zRequestMethod && 'G'==zRequestMethod[0]){
+        /* Apply saved color by defaut for GET requests
+        ** (e.g., an Edit menu link).
+        */
+        zClrFlag = " checked";
+      }
+    }
   }else{
     zClr = "";
     isNew = 1;
@@ -461,7 +477,7 @@ void eventedit_page(void){
       style_header("Error");
       @ Internal error:  Fossil tried to make an invalid artifact for
       @ the edited technote.
-      style_footer();
+      style_finish_page();
       return;
     }
     cgi_redirectf("%R/technote?name=%T", zId);
@@ -497,8 +513,9 @@ void eventedit_page(void){
     @ <blockquote>
     blob_init(&event, 0, 0);
     blob_append(&event, zBody, -1);
+    safe_html_context(DOCSRC_WIKI);
     wiki_render_by_mimetype(&event, zMimetype);
-    @ </blockquote><hr />
+    @ </blockquote><hr>
     blob_reset(&event);
   }
   for(n=2, z=zBody; z[0]; z++){
@@ -508,12 +525,12 @@ void eventedit_page(void){
   if( n>40 ) n = 40;
   @ <form method="post" action="%R/technoteedit"><div>
   login_insert_csrf_secret();
-  @ <input type="hidden" name="name" value="%h(zId)" />
+  @ <input type="hidden" name="name" value="%h(zId)">
   @ <table border="0" cellspacing="10">
 
   @ <tr><th align="right" valign="top">Timestamp (UTC):</th>
   @ <td valign="top">
-  @   <input type="text" name="t" size="25" value="%h(zETime)" />
+  @   <input type="text" name="t" size="25" value="%h(zETime)">
   @ </td></tr>
 
   @ <tr><th align="right" valign="top">Timeline Comment:</th>
@@ -524,19 +541,20 @@ void eventedit_page(void){
 
   @ <tr><th align="right" valign="top">Timeline Background Color:</th>
   @ <td valign="top">
-  @ <input type='checkbox' name='newclr'%s(zClrFlag) />
+  @ <input type='checkbox' name='newclr'%s(zClrFlag)>
   @ Use custom color: \
   @ <input type='color' name='clr' value='%s(zClr[0]?zClr:"#c0f0ff")'>
   @ </td></tr>
 
   @ <tr><th align="right" valign="top">Tags:</th>
   @ <td valign="top">
-  @   <input type="text" name="g" size="40" value="%h(zTags)" />
+  @   <input type="text" name="g" size="40" value="%h(zTags)">
   @ </td></tr>
 
-  @ <tr><th align="right" valign="top">Markup Style:</th>
+  @ <tr><th align="right" valign="top">\
+  @ %z(href("%R/markup_help"))Markup Style</a>:</th>
   @ <td valign="top">
-  mimetype_option_menu(zMimetype);
+  mimetype_option_menu(zMimetype, "mimetype");
   @ </td></tr>
 
   @ <tr><th align="right" valign="top">Page&nbsp;Content:</th>
@@ -546,14 +564,14 @@ void eventedit_page(void){
   @ </td></tr>
 
   @ <tr><td colspan="2">
-  @ <input type="submit" name="cancel" value="Cancel" />
-  @ <input type="submit" name="preview" value="Preview" />
+  @ <input type="submit" name="cancel" value="Cancel">
+  @ <input type="submit" name="preview" value="Preview">
   if( P("preview") ){
-    @ <input type="submit" name="submit" value="Submit" />
+    @ <input type="submit" name="submit" value="Submit">
   }
   @ </td></tr></table>
   @ </div></form>
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -589,7 +607,7 @@ void event_cmd_commit(
 #ifdef FOSSIL_ENABLE_JSON
     g.json.resultCode = FSL_JSON_E_ASSERT;
 #endif
-    fossil_fatal("Internal error: Fossil tried to make an "
+    fossil_panic("Internal error: Fossil tried to make an "
                  "invalid artifact for the technote.");
   }
 }
